<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class ZynithSEOSchema {

    public function __construct() {
        add_action('admin_menu', [$this, 'add_schema_templates_pages'], 99);
    }

    public static function init() {
        return new self();
    }

    public function add_schema_templates_pages() {
        add_submenu_page(
            'zynith-seo-settings',
            __('Schema Templates', ZYNITH_SEO_TEXT_DOMAIN),
            __('Schema Templates', ZYNITH_SEO_TEXT_DOMAIN),
            'manage_options',
            'schema-templates',
            [$this, 'custom_schema_page_html']
        );
    }

    public function custom_schema_page_html() {
        if (isset($_POST['new_template_name']) && isset($_POST['new_template_schema'])) {
            $name = stripslashes(sanitize_text_field($_POST['new_template_name']));
            $schema = stripslashes(sanitize_textarea_field($_POST['new_template_schema']));
            $templates = get_option('custom_schema_templates', array());

            $edit_index = isset($_POST['edit_template_index']) ? intval($_POST['edit_template_index']) : -1;
            if ($edit_index >= 0 && $edit_index < count($templates)) {
                $templates[$edit_index]['name'] = $name;
                $templates[$edit_index]['schema'] = $schema;
            } else {
                $templates[] = [ 'name' => $name, 'schema' => $schema ];
            }

            update_option('custom_schema_templates', $templates);
        }

        $html = '<div class="wrap zynith-wrap"><h1>' . __('Schema Templates', ZYNITH_SEO_TEXT_DOMAIN) . '</h1><p>' . __('Create and save your own schema templates. These will be available for selection on a per page basis.<br/><br/>Please use the following php calls for dynamic insertion in your schema templates below:<br/><br/>Title - &lt;?php echo $title; ?&gt;<br/>Description - &lt;?php echo $meta_description; ?&gt;<br/>Post Date - &lt;?php echo $post_date; ?&gt;<br/>Url - &lt;?php echo $url; ?&gt;<br/>Author Name - &lt;?php echo $author_name; ?&gt;<br/>Featured Image - &lt;?php echo get_the_post_thumbnail_url($object_id); ?&gt;<br/>', ZYNITH_SEO_TEXT_DOMAIN) . '</p><div class="postbox zynith_seo"><div class="inside"><h2>' . __('New Schema Template', ZYNITH_SEO_TEXT_DOMAIN) . '</h2><form id="form_schema_template" method="POST" onsubmit="handleFormSubmit(event)"><div class="div_form_col"><label for="new_template_name">' . __('Template Name', ZYNITH_SEO_TEXT_DOMAIN) . '</label><input type="text" id="new_template_name" name="new_template_name" placeholder="' . __('Template Name', ZYNITH_SEO_TEXT_DOMAIN) . '" required /></div><!-- End: .div_form_col --><div class="div_form_col"><label for="new_template_schema">' . __('Schema Template', ZYNITH_SEO_TEXT_DOMAIN) . '</label><textarea id="new_template_schema" name="new_template_schema" rows="9" placeholder="' . __('Schema Template', ZYNITH_SEO_TEXT_DOMAIN) . '" required></textarea></div><!-- End: .div_form_col --><div class="div_button_container"><button type="submit">' . __('Save Template', ZYNITH_SEO_TEXT_DOMAIN) . '</button></div><!-- End: .div_button_container --></form></div><!-- End: .inside --></div><!-- End: .postbox -->';

        $templates = get_option('custom_schema_templates', array());
        if (!empty($templates)) {
            $html .= '<div class="postbox zynith_seo"><div class="inside"><h2>' . __('Existing Schema Templates', ZYNITH_SEO_TEXT_DOMAIN) . '</h2><div class="div_row"><select id="existing_template_selector" name="existing_template_selector" onchange="populate_template_fields()"><option value="">- ' . __('Select Template', ZYNITH_SEO_TEXT_DOMAIN) . ' -</option>';

            foreach ($templates as $index => $template) {
                $html .= '<option value="' . $index . '">' . esc_html($template['name']) . '</option>';
            }

            $encoded_templates = json_encode($templates);
            $html .= '</select></div><!-- End: .div_row --><div class="div_button_container"><button type="button" onclick="delete_template()">Delete</button></div><!-- End: .div_button_container --></div><!-- End: .inside --></div><!-- End: .postbox -->
            <script>
                function populate_template_fields() {
                    let index = document.getElementById("existing_template_selector").value;
                    let templates = ' . $encoded_templates . ';
                    if (index !== "") {
                        document.getElementById("new_template_name").value = templates[index].name;
                        document.getElementById("new_template_schema").value = templates[index].schema;

                        // Add the hidden input field for the template index
                        let form = document.getElementById("form_schema_template");
                        let editIndexInput = document.getElementById("edit_template_index");
                        if (!editIndexInput) {
                            editIndexInput = document.createElement("input");
                            editIndexInput.type = "hidden";
                            editIndexInput.id = "edit_template_index";
                            editIndexInput.name = "edit_template_index";
                            form.appendChild(editIndexInput);
                        }
                        editIndexInput.value = index;
                    }
                    else {
                        // Reset the form fields if no template is selected
                        document.getElementById("new_template_name").value = "";
                        document.getElementById("new_template_schema").value = "";
                        let editIndexInput = document.getElementById("edit_template_index");
                        if (editIndexInput) {
                            form.removeChild(editIndexInput);
                        }
                    }
                }

                function delete_template() {
                    let index = document.getElementById("existing_template_selector").value;
                    let deleteForm = document.getElementById("delete_template_form");

                    if (deleteForm) {
                        deleteForm.parentNode.removeChild(deleteForm);
                    }

                    let form = document.createElement("form");
                    form.id = "delete_template_form";
                    form.method = "POST";

                    let input = document.createElement("input");
                    input.type = "hidden";
                    input.name = "delete_template_index";
                    input.value = index;
                    form.appendChild(input);
                    document.body.appendChild(form);
                    form.submit();
                }
            </script>';

            if (isset($_POST['delete_template_index'])) {
                $index = intval($_POST['delete_template_index']);
                array_splice($templates, $index, 1);
                update_option('custom_schema_templates', $templates);
                echo '<script>location.reload();</script>';
            }
        }

        $html .= '</div><!-- End: .wrap -->';
        echo $html;
    }
}
