<?php
if (!defined('ABSPATH')) { exit; }

class ZynithSEOSettings {
	public $option_prefix = 'zynith_';
	public $settings = [];
	public $default_toc_css = '.zynith-toc {
	border: 1px solid gray;
    padding: 20px;
}
.zynith-toc ul {
	margin: 5px 0;
	padding-left: 0;
	list-style-type: none;
}
.zynith-toc li {
	margin-bottom: 0.1em;
}
.zynith-toc .toc-h1 a {
	font-weight: bold;
    font-size: 1.6em;
}
.zynith-toc .toc-h2 { padding-left: 20px; }
.zynith-toc .toc-h3 { padding-left: 40px; }
.zynith-toc .toc-h4 { padding-left: 60px; }
.zynith-toc .toc-h5 { padding-left: 80px; }
.zynith-toc .toc-h6 { padding-left: 100px; }';
	
	public function __construct() {
		add_action( 'plugins_loaded', [ $this, 'plugin_loaded' ]);
		add_action( 'admin_menu', [ $this, 'pre_render_settings' ] );
		add_action( 'admin_init', [ $this, 'register_settings' ] );
		add_action( 'admin_menu', [ $this, 'add_settings_menus' ] );
		add_action( 'admin_menu', [ $this, 'rearrange_admin_menu' ], 999 );
		add_action( 'admin_init', [ $this, 'export_seo_data' ] );
		add_action( 'admin_init', [ $this, 'import_seo_data' ] );
		add_action( 'admin_init', [ $this, 'purge_option' ] );
		add_action( 'admin_init', [ $this, 'purge_zynith_meta' ] );
		add_action( 'settings_page_zynith-seo-settings', [ $this, 'plugin_importer' ] );
		add_action( 'settings_page_zynith-seo-settings', [ $this, 'add_purge_option' ] );
		add_action( 'admin_notices', [ $this, 'show_updated_notice' ] );
		add_action( 'admin_enqueue_scripts', [ $this, 'enqueue_scripts' ] );
		add_action( 'admin_post_purge_zynith_meta_keys', [ $this, 'purge_zynith_meta_keys' ] );
		add_action( 'admin_notices', [ $this, 'show_sitemap_xml_warnings' ] );
		add_action( 'admin_init', [ $this, 'sitemap_xml_actions' ] );
	}

	public static function init() {
        return new self();
    }
	
	function plugin_loaded() {
		global $wpdb;
		$table_name = $wpdb -> prefix . 'zynith_redirects';
		if ($wpdb -> get_var("SHOW TABLES LIKE '$table_name'") == $table_name) {
			$wpdb -> query("DROP TABLE {$table_name}");
		}
		
		$table_name = $wpdb -> prefix . 'zynith_404_log';
		if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
			$charset_collate = $wpdb -> get_charset_collate();

			$sql = "CREATE TABLE $table_name (
				id mediumint(9) NOT NULL AUTO_INCREMENT,
				url varchar(255) NOT NULL,
				user_agent varchar(255) NOT NULL,
				timestamp timestamp DEFAULT CURRENT_TIMESTAMP NOT NULL,
				PRIMARY KEY  (id)
			) $charset_collate;";

			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
			dbDelta($sql);
		}
	}
	
	public function pre_render_settings() {
		$this->settings = $this->get_settings();
	}

	public static function get_sitemap_name() {
        $filename = get_option( 'zynith_custom_sitemap_filename', 'sitemap.xml' );
		if ( empty( $filename ) ) {
			$filename = 'sitemap.xml';
		}

		if ( substr( $filename, -4 ) !== '.xml' ) {
			$filename .= '.xml';
		}

		$file_info = pathinfo( $filename );
		$dir = rtrim( $file_info['dirname'], '/' ) . '/';
		$name = $file_info['filename'];
		$ext = $file_info['extension'];

		return sanitize_title( $dir . $name ) . '.' . $ext;
    }
	
	public function get_settings() {
		$category_example1 = '<code>' . home_url( '/category/uncategorized/' ) . '</code>';
		$category_example2 = '<code>' . home_url( '/uncategorized/' ) . '</code>';
		$default_css = $this->default_toc_css;
		
		$settings['zynith-seo-settings'] = [
			'full_title' => __('Zynith SEO Settings', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __('Zynith SEO', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __('Metabox Settings', ZYNITH_SEO_TEXT_DOMAIN),
					'fields' => [
						[
							'id' => 'signals_instructions',
							'type' => 'message',
							'label' => __( 'Instructions:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Select whether you wish to hide specific signals on each page/post.', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => ''
						],
						[
							'id' => 'seo_signals_enabled',
							'type' => 'checkbox_group',
							'label' => __('Show SEO signals:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => [
								'seo_signals_fk_grade_enabled' => [
									'label' => __( 'Flesch-Kincaid Grade Level', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_content_length_enabled' => [
									'label' => __( 'Content Length', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_html_ratio_enabled' => [
									'label' => __( 'HTML to Text Ratio', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_broken_links_enabled' => [
									'label' => __( 'Broken Links', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_internal_links_enabled' => [
									'label' => __( 'Internal Links', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_external_links_enabled' => [
									'label' => __( 'External Links', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								],
								'seo_signals_ai_content_enabled' => [
									'label' => __( 'AI Generated Content', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN )
								]
							],
							'default' => null
						],
						[
							'id' => 'show_quick_ryter',
							'type' => 'checkbox',
							'label' => __( 'Quick Ryter', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Show Zynith Quick Ryter for Posts. <i>(You will need an OpenAI API Key.)</i>', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => ''
						],
					]
				],
				[
					'title' => __( 'Table of Contents', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'toc_implementation',
							'type' => 'message',
							'label' => __( 'Instructions:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => sprintf(
								__( 'You can add a Table of Contents to any content by using the shortcode: %s[zynith-toc]%s%sTo implement the Table of Contents in your templates, you can use this PHP function: %s', ZYNITH_SEO_TEXT_DOMAIN ),
								'<code>',
								'</code>',
								'<br>',
								'<code>&lt;?php zynith_toc(); ?&gt;</code>'
							),
							'default' => '',
						],
						[
							'id' => 'toc_heading_1_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 1:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 0,
						],
						[
							'id' => 'toc_heading_2_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 2:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 1,
						],
						[
							'id' => 'toc_heading_3_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 3:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 0,
						],
						[
							'id' => 'toc_heading_4_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 4:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 0,
						],
						[
							'id' => 'toc_heading_5_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 5:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 0,
						],
						[
							'id' => 'toc_heading_6_enabled',
							'type' => 'checkbox',
							'label' => __('Show Heading 6:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => 0,
						],
						[
							'id' => 'toc_css',
							'type' => 'textarea',
							'label' => __( 'Customize CSS:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => $default_css,
						],
						[
							'id' => 'toc_restore_css',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 500">' . sprintf( __( "%s to restore Zynith TOC CSS. Click 'Save Changes' below to keep the restored changes.", ZYNITH_SEO_TEXT_DOMAIN ), '<a href="#zynith_toc_css" class="zynith-button-small" onClick="restore_toc_css();">' . __( 'Click here', ZYNITH_SEO_TEXT_DOMAIN ) . '</a>' ) . '</span>',
							'default' => '',
						]
					],
				],
				[
					'title' => __( 'Breadcrumbs', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'bc_implementation',
							'type' => 'message',
							'label' => __( 'Instructions:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => sprintf(
								__( 'You can add breadcrumbs to any content by using the shortcode: %s[zynith-breadcrumbs]%s%sTo implement breadcrumbs in your templates, you can use this PHP function: %s', ZYNITH_SEO_TEXT_DOMAIN ),
								'<code>',
								'</code>',
								'<br>',
								'<code>&lt;?php zynith_seo_breadcrumbs(); ?&gt;</code>'
							),
							'default' => '',
						],
						[
							'id' => 'add_breadcrumbs_schema',
							'type' => 'checkbox',
							'label' => __( 'Add breadcrumbs schema:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Automatically add breadcrumbs schema when using breadcrumbs', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_left' => __( 'Disable', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Enable', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						]
					],
				],
				[
					'title' => __( 'Styles', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'style_select',
							'type' => 'select',
							'label' => __( 'Select Zynith style:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => [
								'default' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
								'dark' => __( 'Dark Mode', ZYNITH_SEO_TEXT_DOMAIN ),
								'synthwave' => __( 'Synth Wave', ZYNITH_SEO_TEXT_DOMAIN ),
								'zealot' => __( 'Zealot', ZYNITH_SEO_TEXT_DOMAIN ),
							],
							'default' => 'default',
						],
					],
				],
				[
					'title' => __( 'WordPress Mods', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'remove_canonical',
							'type' => 'checkbox',
							'label' => __( 'Remove Canonical Tag:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Allow an alternate plugin to add the canonical tag.', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_left' => __( 'On', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Off', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'remove_category_base',
							'type' => 'checkbox',
							'label' => __( 'Category Base:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => sprintf( __( 'Remove the category base from URLs. Eg. %s becomes %s', ZYNITH_SEO_TEXT_DOMAIN ), $category_example1, $category_example2 ),
							'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Remove', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'notes',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 400; color: #FF0020">' . __( 'Removing the category base is generally not recommended, as it can cause conflicts with page URLs and end up damaging your SEO.', ZYNITH_SEO_TEXT_DOMAIN ) . '</span>',
							'default' => '',
						],
						[
							'id' => 'remove_rss_feeds',
							'type' => 'checkbox',
							'label' => __( 'RSS Feeds:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Remove default RSS feeds', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Remove', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'remove_generator_tags',
							'type' => 'checkbox',
							'label' => __( 'Generator Tags:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'Remove generator tags from the source code.', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Remove', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'custom_sitemap_filename',
							'type' => 'text',
							'label' => __( 'Sitemap Filename:', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => 'sitemap.xml',
							'required' => true,
							'default' => self::get_sitemap_name(),
						],
						[
							'id' => 'sitemap_notes',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 500">' . sprintf( __( 'View your sitemap at %s', ZYNITH_SEO_TEXT_DOMAIN ), '<a href="' . home_url( self::get_sitemap_name() ) . '" target="_blank">' . home_url( self::get_sitemap_name() ) . '</a>' ) . '</span>',
							'default' => '',
						],
						[
							'id' => 'sitemap_actions',
							'type' => 'message',
							'label' => '',
							'description' => '<span style="font-weight: 500">' . sprintf( __( "If you're experiencing issues with the sitemap or changed the sitemap name, please %s to manually regenerate it.", ZYNITH_SEO_TEXT_DOMAIN ), '<a href="' . add_query_arg( [ 'page' => 'zynith-seo-settings', 'action' => 'regenerate-sitemap-xml', '_wpnonce' => wp_create_nonce( 'regenerate-sitemap-xml' ) ], admin_url() ) . '" class="zynith-button-small">' . __( 'click here', ZYNITH_SEO_TEXT_DOMAIN ) . '</a>' ) . '</span>',
							'default' => '',
						],
					],
				],
				[
					'title' => __( 'Homepage SEO Settings', ZYNITH_SEO_TEXT_DOMAIN ),
					'description' => [ $this, 'home_page_seo_description' ],
					'fields' => [
						[
							'id' => 'homepage_meta_title',
							'type' => 'text',
							'label' => __( 'Meta Title:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => '',
						],
						[
							'id' => 'homepage_meta_description',
							'type' => 'textarea',
							'label' => __( 'Meta Description:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => '',
							'default' => '',
						],
					],
				]
			]
		];

		$settings['zynith-seo-business-info'] = [
			'full_title' => __( 'Business Details', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __( 'Business Details', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __( 'General', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'business_name',
							'type' => 'text',
							'label' => __( 'Business Name', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'business_email',
							'type' => 'text',
							'label' => __( 'Business Email', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'phone_number',
							'type' => 'text',
							'label' => __( 'Phone Number', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'street_address',
							'type' => 'text',
							'label' => __( 'Street Address', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'address_locality',
							'type' => 'text',
							'label' => __( 'Locality', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'address_region',
							'type' => 'text',
							'label' => __( 'Region', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'postal_code',
							'type' => 'text',
							'label' => __( 'Postal Code', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'address_country',
							'type' => 'text',
							'label' => __( 'Country', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'logo',
							'type' => 'image',
							'label' => __('Logo', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
					],
				],
				[
					'title' => __( 'Social', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'default_og_image',
							'type' => 'image',
							'label' => __( 'Default Open Graph Image', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
					],
				],
			]
		];
		
		$settings['zynith-seo-gtm'] = [
			'full_title' => __( 'Zynith SEO Scripts', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __( 'Scripts', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __( 'Scripts (e.g. Google Tag Manager, Google Analytics, Facebook Pixel, etc.)', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'gtm_header_script',
							'type' => 'textarea',
							'label' => __( 'Head Script', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
						[
							'id' => 'gtm_body_script',
							'type' => 'textarea',
							'label' => __( 'Body Script', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
						],
					],
				],
			]
		];

		$settings['zynith-seo-robots-txt'] = [
			'full_title' => __( 'Zynith SEO Robots.txt', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __( 'Robots.txt', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __( 'SEO Robots.txt Editor', ZYNITH_SEO_TEXT_DOMAIN ),
					'description' => [ $this, 'robots_txt_description' ],
					'fields' => [
						[
							'id' => 'dynamic_robots_txt',
							'type' => 'checkbox',
							'label' => __( 'Use dynamic robots.txt:', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => sprintf( __( '%sWarning:%s Using a dynamic robots.txt will delete the static robots.txt file when you save the settings!%sWe will automatically add your sitemap and a few other settings using dynamic robots.txt.%s', ZYNITH_SEO_TEXT_DOMAIN ), '<br><p class="description" style="color: red !important; font-size: 14px;display: inline-block;padding: 0.5rem 0.8rem;border: 2px solid darkred; border-radius: 8px; margin-top: 6px;"><strong style="text-decoration:underline">', '</strong>', '<br>', '</p>' ),
							'handle_left' => __( 'Static', ZYNITH_SEO_TEXT_DOMAIN ),
							'handle_right' => __( 'Dynamic', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => 0,
						],
					],
				],
			]
		];

		$use_dynamic_robots_txt = (bool) get_option( 'zynith_dynamic_robots_txt', false );

		if ( $use_dynamic_robots_txt ) {
			$settings['zynith-seo-robots-txt']['groups'][0]['description'] = '';
			$settings['zynith-seo-robots-txt']['groups'][0]['fields'][0]['description'] = __( 'We will automatically add your sitemap and a few other settings using dynamic robots.txt.', ZYNITH_SEO_TEXT_DOMAIN );
			$settings['zynith-seo-robots-txt']['groups'][0]['fields'][] = [
				'id' => 'robots_txt_remove_wp_default',
				'type' => 'checkbox',
				'label' => __( 'Default robots.txt output', ZYNITH_SEO_TEXT_DOMAIN ),
				'description' => __( 'Remove the default robots.txt output added by WordPress in the dynamic robots.txt', ZYNITH_SEO_TEXT_DOMAIN ),
				'default' => 1,
				'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
				'handle_right' => __( 'Remove', ZYNITH_SEO_TEXT_DOMAIN ),
			];
			$settings['zynith-seo-robots-txt']['groups'][0]['fields'][] = [
				'id' => 'robots_txt_allow_pdf_index',
				'type' => 'checkbox',
				'label' => __( 'PDF indexing', ZYNITH_SEO_TEXT_DOMAIN ),
				'description' => __( 'Allow indexing of your PDF files', ZYNITH_SEO_TEXT_DOMAIN ),
				'default' => 1,
				'handle_left' => __( 'Block', ZYNITH_SEO_TEXT_DOMAIN ),
				'handle_right' => __( 'Allow', ZYNITH_SEO_TEXT_DOMAIN ),
			];
		}
		else {
			$settings['zynith-seo-robots-txt']['groups'][0]['fields'][] = [
				'id' => 'robots_txt_content',
				'type' => 'textarea',
				'label' => __( 'SEO Robots.txt Editor', ZYNITH_SEO_TEXT_DOMAIN ),
				'default' => '',
			];
		}

		$settings['zynith-seo-robots-meta'] = [
			'full_title' => __( 'Zynith SEO Robots Meta', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __( 'Robots Meta', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __( 'Global noindex', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'noindex_post_types',
							'type' => 'checkbox_group',
							'label' => __( 'Apply noindex to post types:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => ZynithSEOPostTypes::get_post_types(),
							'default' => '',
						],
						[
							'id' => 'noindex_taxonomies',
							'type' => 'checkbox_group',
							'label' => __( 'Apply noindex to taxonomies:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => ZynithSEOTaxonomies::get_taxonomies(),
							'default' => [ 'product_shipping_class' ],
						],
						[
							'id' => 'noindex_archives',
							'type' => 'checkbox_group',
							'label' => __( 'Apply noindex to archives:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => [
								'author' => [
									'label' => __( 'Author', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
								'date' => [
									'label' => __( 'Date', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
								'search' => [
									'label' => __( 'Search', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
							],
							'default' => [ 'search' ],
						],
					],
				],
				[
					'title' => __( 'Global nofollow', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'nofollow_post_types',
							'type' => 'checkbox_group',
							'label' => __( 'Apply nofollow to post types:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => ZynithSEOPostTypes::get_post_types(),
							'default' => '',
						],
						[
							'id' => 'nofollow_taxonomies',
							'type' => 'checkbox_group',
							'label' => __( 'Apply nofollow to taxonomies:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => ZynithSEOTaxonomies::get_taxonomies(),
							'default' => '',
						],
						[
							'id' => 'nofollow_archives',
							'type' => 'checkbox_group',
							'label' => __( 'Apply nofollow to archives:', ZYNITH_SEO_TEXT_DOMAIN ),
							'options' => [
								'author' => [
									'label' => __( 'Author', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
								'date' => [
									'label' => __( 'Date', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
								'search' => [
									'label' => __( 'Search', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_left' => __( 'Default', ZYNITH_SEO_TEXT_DOMAIN ),
									'handle_right' => __( 'Apply', ZYNITH_SEO_TEXT_DOMAIN ),
								],
							],
							'default' => '',
						],
					],
				],
			]
		];

		if ( ! get_option( 'blog_public' ) ) {
			foreach ( $settings['zynith-seo-robots-meta']['groups'] as $index => $group ) {
				$settings['zynith-seo-robots-meta']['groups'][ $index ]['description'] = [ $this, 'robots_meta_description' ];
			}
		}

		// License settings
		$settings['zynith-seo-license'] = [
			'full_title' => __( 'Zynith SEO License and API', ZYNITH_SEO_TEXT_DOMAIN ),
			'menu_title' => __( 'License/API Keys', ZYNITH_SEO_TEXT_DOMAIN ),
			'groups' => [
				[
					'title' => __( 'License Key', ZYNITH_SEO_TEXT_DOMAIN ),
					'fields' => [
						[
							'id' => 'license_key',
							'type' => 'password',
							'label' => __( 'Zynith SEO License Key', ZYNITH_SEO_TEXT_DOMAIN ),
							'description' => __( 'The license key allows you to verify your purchase and receive updates for the plugin.', ZYNITH_SEO_TEXT_DOMAIN ),
							'default' => '',
							'purge_button' => true,
							'purge_button_text' => __( 'Remove Key', ZYNITH_SEO_TEXT_DOMAIN ),
							'confirm_dialog' => __( 'Are you sure you want to remove the license key? You will no longer receive plugin updates!', ZYNITH_SEO_TEXT_DOMAIN ),
						],
						[
							'id' => 'quick_ryter_api_key',
							'type' => 'password',
							'label' => __('OpenAI API Key:', ZYNITH_SEO_TEXT_DOMAIN),
							'description' => sprintf( __( "By providing an OpenAI API Key, you will gain access to additional AI features, including the Quick Ryter and AI Schema generator.<br />If you don't have an API key, please obtain one from %sOpenAI%s", ZYNITH_SEO_TEXT_DOMAIN), '<a href="https://platform.openai.com/" target="_blank">', '</a>.<br /><br /><strong>Warning:</strong> The AI features in this plugin may result in additional charges based on a per-token rate set by OpenAI. Zynith does not cover these charges, and it is your responsibility. Please use these features with caution and be aware of the potential cost.' ),
							'default' => '',
							'purge_button' => true,
							'purge_button_text' => __('Remove Key', ZYNITH_SEO_TEXT_DOMAIN),
							'confirm_dialog' => __('Are you sure you want to remove the API key?', ZYNITH_SEO_TEXT_DOMAIN),
						],
					],
				],
			]
		];
		
		return $settings;
	}

	public function robots_meta_description() {
		echo '<p><strong style="font-size: 14px;display: inline-block;padding: 0.5rem 0.8rem;border: 2px solid darkred;border-radius: 8px;background: #fff;">' . sprintf( __( 'Your website discourages search engines from indexing. To make these settings work, please change the search engine visibility %shere%s.', ZYNITH_SEO_TEXT_DOMAIN ), '<a href="' . admin_url( 'options-reading.php' ) . '" style="color: #cc001b;">', '</a>' ) . '</strong></p>';
	}
	
	public function register_settings() {
		$this->register_settings_fields( $this->settings );
	}
	
	public function add_settings_menus() {
		$counter = 1;
		$main_settings = '';
		
		foreach ($this->settings as $key => $data ) {
			$full_title = $data['full_title'];
			$menu_title = $data['menu_title'];
			
			if ($counter === 1) {
				$main_settings = $key;
				add_menu_page($full_title, $menu_title, 'manage_options', $key, [$this, 'render_zynith_seo_settings_page'], 'dashicons-search', 90);
			}
			else {
				add_submenu_page($main_settings, $full_title, $menu_title, 'manage_options', $key, [$this, 'render_zynith_seo_settings_page']);
			}
			
			// Set the menu name for the first submenu item
			global $submenu;
			if (isset($submenu[$main_settings])) {
				$submenu[$main_settings][0][0] = 'Settings';
			}
			
			++$counter;
		}
		
	}

	public function rearrange_admin_menu() {
		global $submenu;

		if ( empty( $submenu['zynith-seo-settings'] ) ) {
			return;
		}

		$new_structure = [];
		$positions = [
			'zynith-seo-settings' => 0,
			'zynith-seo-business-info' => 1,
			'schema-templates' => 2,
			'zynith-seo-gtm' => 3,
			'zynith-seo-robots-txt' => 4,
			'zynith-seo-robots-meta' => 5,
			'search-and-replace' => 6,
			'404-monitor-settings' => 7,
			'zynith-seo-license' => 8
		];

		$fallback_counter = count( $positions );

		foreach ( $submenu['zynith-seo-settings'] as $item ) {
			$position = $positions[ $item[2] ] ?? $fallback_counter++;
			$new_structure[ $position ] = $item;
		}
		
		ksort( $new_structure );
		$submenu['zynith-seo-settings'] = $new_structure;
	}
	/*
	public function restore_toc_css() {
		$action = filter_input( INPUT_POST, 'action' );
		if ( $action !== 'restore-toc-css' ) {
			return;
		}
		//$page = filter_input( INPUT_GET, 'page' );
		//$action_url = admin_url( 'admin.php?page=' . $page );
		$nonce = filter_input( INPUT_GET, '_wpnonce' );
		if ( ! wp_verify_nonce( $nonce, $action ) ) {
			wp_die( __( 'Restore CSS failed.', ZYNITH_SEO_TEXT_DOMAIN ) );
		}
		$css = $this->default_toc_css;
		update_option('zynith_toc_css', $css);
		
		$referrer = htmlspecialchars( filter_input( INPUT_SERVER, 'HTTP_REFERER' ) );
		wp_safe_redirect($referrer);
		exit;
	}
	*/
	public function purge_option() {
		$action = filter_input( INPUT_POST, 'action' );

		if ( $action !== 'purge-option' ) {
			return;
		}

		$option = htmlspecialchars( filter_input( INPUT_POST, 'option' ) );
		$deleted = (bool) delete_option( $option );
		$message = ! $deleted ? __( 'We were unable to remove the value.', ZYNITH_SEO_TEXT_DOMAIN ) : '';

		wp_send_json( [ 'success' => $deleted, 'message' => $message ] );
	}
	
	public function sitemap_xml_actions() {
		$action = filter_input( INPUT_GET, 'action' );
				
		if ( $action === 'dismiss-sitemap-xml' ) {
			$nonce = filter_input( INPUT_GET, '_wpnonce' );
			$referrer = htmlspecialchars( filter_input( INPUT_SERVER, 'HTTP_REFERER' ) );
			if ( ! wp_verify_nonce( $nonce, $action ) ) {
				wp_die( __( 'Verification failed.', ZYNITH_SEO_TEXT_DOMAIN ) );
			}
			
			delete_option( 'sitemap_xml_error' );
			wp_redirect( $referrer );
		}

		if ( $action === 'regenerate-sitemap-xml' ) {
			$nonce = filter_input( INPUT_GET, '_wpnonce' );
			$referrer = htmlspecialchars( filter_input( INPUT_SERVER, 'HTTP_REFERER' ) );
			if ( ! wp_verify_nonce( $nonce, $action ) ) {
				wp_die( __( 'Verification failed.', ZYNITH_SEO_TEXT_DOMAIN ) );
			}

			( new ZynithSEOSitemap() )->generate_sitemap();
			
			wp_redirect( add_query_arg( 'sitemap-generated', 'true', $referrer ) );
		}
		
	}
	
	public function purge_zynith_meta() {
		$action = filter_input( INPUT_GET, 'action' );
		if ( $action !== 'purge-zynith' ) {
			return;
		}

		$nonce = filter_input( INPUT_GET, '_wpnonce' );
		if ( ! wp_verify_nonce( $nonce, 'purge-zynith' ) ) {
			wp_die( __( 'Verification failed.', ZYNITH_SEO_TEXT_DOMAIN ) );
		}

		global $wpdb;

		$meta_keys = [
			'_custom_meta_title',
			'_custom_meta_description',
			'_custom_noindex',
			'_custom_nofollow',
			'_custom_target_keyword',
			'_custom_schema',
		];

		foreach ($meta_keys as $meta_key) {
			$wpdb->delete( $wpdb->postmeta, [ 'meta_key' => $meta_key ] );
		}

		$page = filter_input( INPUT_GET, 'page' );
		$redirect_url = add_query_arg( 'zynith-purged', 'true', admin_url( 'admin.php?page=' . $page ) );

		wp_redirect( $redirect_url );
		exit;
	}
	
public function plugin_importer() {
    ?>
    <hr />
    <div class="postbox zynith_seo">
        <div class="inside">
            <h2>Import / Export SEO</h2>
            <form method="post" action="" enctype="multipart/form-data">
                <table class="form-table" role="presentation">
                    <tbody>
                        <tr>
                            <th scope="row">Select SEO Plugin:</th>
                            <td>
                                <select name="seo_plugin" id="seo_plugin" class="regular-text styled-select">
                                    <option value="rankmath">Rank Math</option>
                                    <option value="yoast">Yoast SEO</option>
                                    <option value="zynith">Zynith SEO</option>
                                </select>
                            </td>
                        </tr>
                        <tr>
                            <th scope="row">Upload CSV File:</th>
                            <td>
                                <input type="file" name="csv_file" id="csv_file" />
                            </td>
                        </tr>
                        <tr>
                            <th scope="row"></th>
                            <td>
                                <input type="submit" name="import_seo" class="button button-primary" value="Import" />
                                <input type="submit" name="export_seo" class="button button-primary" value="Export" />
                            </td>
                        </tr>
                    </tbody>
                </table>
            </form>
        </div>
    </div>
    <?php
}

	
	public function export_seo_data() {
    if (isset($_POST['export_seo'])) {
        $seo_plugin = sanitize_text_field($_POST['seo_plugin']);
        $meta_keys = array();

        // Determine the meta keys based on the selected plugin
        switch ($seo_plugin) {
            case 'rankmath':
                $meta_keys = ['rank_math_title', 'rank_math_description', 'rank_math_schema', 'rank_math_noindex', 'rank_math_nofollow', 'rank_math_focus_keyword'];
                break;
            case 'yoast':
                $meta_keys = ['_yoast_wpseo_title', '_yoast_wpseo_metadesc', '_yoast_wpseo_schema', '_yoast_wpseo_noindex', '_yoast_wpseo_nofollow', '_yoast_wpseo_focuskw'];
                break;
            case 'zynith':
                $meta_keys = ['_custom_meta_title', '_custom_meta_description', '_custom_schema', '_custom_noindex', '_custom_nofollow', '_custom_target_keyword'];
                break;
        }

        $filename = 'seo_data.csv';
        header('Content-Type: text/csv');
        header('Content-Disposition: attachment; filename="' . $filename . '"');
        $output = fopen('php://output', 'w');
        fputcsv($output, array_merge(['Post/Page ID'], $meta_keys));

        $args = array(
            'post_type'      => array('post', 'page'),
            'posts_per_page' => -1,
            'post_status'    => 'publish'
        );
        $seo_query = new WP_Query($args);

        if ($seo_query->have_posts()) {
            while ($seo_query->have_posts()) {
                $seo_query->the_post();
                $post_id = get_the_ID();
                $row = array($post_id);

                foreach ($meta_keys as $key) {
                    $meta_value = get_post_meta($post_id, $key, true);
                    $row[] = $meta_value;
                }

                fputcsv($output, $row);
            }
            wp_reset_postdata();
        }

        fclose($output);
        exit;
    }
}

	
	public function import_seo_data() {
    
		if (isset($_POST['import_seo'])) {
			$csv_file = $_FILES['csv_file'];

			if ($csv_file['error'] === UPLOAD_ERR_OK) {
				$csv_data = array_map('str_getcsv', file($csv_file['tmp_name']));
				array_shift($csv_data); // Remove the header row

				foreach ($csv_data as $row) {
					$post_id = (int) $row[0]; // Post/Page ID

					// Extracting data from CSV row
					$meta_title = isset($row[1]) ? $row[1] : '';
					$meta_description = isset($row[2]) ? $row[2] : '';
					$noindex = isset($row[4]) ? $row[4] : '';
					$nofollow = isset($row[5]) ? $row[5] : '';

					// Update post meta regardless of the SEO plugin
					update_post_meta($post_id, '_custom_meta_title', $meta_title);
					update_post_meta($post_id, '_custom_meta_description', $meta_description);
					update_post_meta($post_id, '_custom_noindex', $noindex);
					update_post_meta($post_id, '_custom_nofollow', $nofollow);
				}
			}
		}
	}
	
	
	
	public function add_purge_option() {
		$page = filter_input( INPUT_GET, 'page' );
		$action_url = admin_url( 'admin.php?page=' . $page );
		
		if ( ! empty( $page ) ) {
			$action_url .= '&action=purge-zynith';
			$action_url = add_query_arg( '_wpnonce', wp_create_nonce( 'purge-zynith' ), $action_url );
		}
		
		echo	'<hr><h2>' . __( 'Purge Zynith SEO Data', ZYNITH_SEO_TEXT_DOMAIN ) . '</h2><p style="margin: 0; color: #6b0006; font-size: 15px; letter-spacing: 1px; line-height: 15px;"><span style="font-weight: 500;">Warning: </span>Pressing this button will delete all data and database tables created with Zynith.</p><a href="' . $action_url . '" id="purge_zynith" class="button button-link-delete delete" onclick="return confirm(\'' . __( 'Are you sure you want to purge Zynith SEO meta data? This cannot be undone.', ZYNITH_SEO_TEXT_DOMAIN ) . '\')">' . __( 'Purge Zynith SEO Data', ZYNITH_SEO_TEXT_DOMAIN ) . '</a>';
	}
	
	public function robots_txt_description() {
		echo '<p>' . __( 'Recommended syntax for Robots.txt is the following:', ZYNITH_SEO_TEXT_DOMAIN ) . '</p>
		<p style="padding-left: 10px; border-left: 2px solid #2d80ea;">User-agent: *<br>Disallow: <br>Sitemap: ' . home_url( $this->get_sitemap_name() ) . '</p>';
	}

	public function home_page_seo_description() {
		echo '<p>' . sprintf( __( 'This section is only for the blog home/posts page. If using a static posts page, these settings will used as fallback if no meta data is present on the posts page itself. You can view and edit the posts page settings %shere%s.', ZYNITH_SEO_TEXT_DOMAIN ), '<a href="' . admin_url( 'options-reading.php' ) . '">', '</a>' ) . '</p>';
	}
	
	public function render_zynith_seo_settings_page() {
		$current_page = htmlspecialchars( filter_input( INPUT_GET, 'page' ) ?? '' );
		
		if (!empty( $this->settings[ $current_page ] ) ) {
			$settings = $this->settings[ $current_page ];
			echo '<div class="wrap zynith-wrap"> <h1>' . $settings['full_title'] . '</h1><form method="post" action="options.php" novalidate="novalidate">';
			settings_fields( $current_page );
			do_settings_sections( $current_page );
			submit_button();
			echo '</form>';
			do_action( 'settings_page_' . $current_page );
			echo '</div>';
		}
	}
	
	public function register_settings_fields( $settings ) {
		
		foreach ( $settings as $location => $data ) {
			foreach ( $data['groups'] as $key => $group ) {
				$group_title = $group['title'] ?? '';
				$group_id = sanitize_title( $group_title );
				$description_callback = $group['description'] ?? '__return_empty_string';
				$fields = $group['fields'];
				
				add_settings_section( $group_id, $group_title, $description_callback, $location, [ 'before_section' => '<div class="postbox zynith_seo"><div class="inside">', 'after_section' => '</div></div>' ] );
				
				foreach ( $fields as $field ) {
					if (! is_array($field)) {
						continue;
					}
					
					if ( empty( $field['id'] ) ) {
						$id = md5( ( ! empty( $field['description'] ) ? $field['description'] : $field['type'] ) . microtime( true ) );
						$field['id'] = $this->option_prefix . $id;
					} else {
						$field['id'] = $this->option_prefix . $field['id'];
					}
					
					register_setting( $location, $field['id'] );
					add_settings_field( $field['id'], $field['label'] ?? '', [ $this, 'render_settings_field' ], $location, $group_id, $field );
				}
			}
		}
	}
	
	public function render_settings_field( $field ) {
		switch ( $field['type'] ) {
			case 'checkbox':
				echo '<fieldset>';
				$default = get_option( $field['id'], $field['default'] );
				$disabled = (bool) ( $field['disabled'] ?? false );
				echo '<label for="' . esc_attr( $field['id'] ) . '" class="check-switch">';
				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="checkbox" value="1"' . ( ! empty( $default ) ? ' checked' : '' ) . ( ! empty( $disabled ) ? ' disabled' : '' ) . '><span class="slider"></span></label>' . $field['description'];
				echo '</fieldset>';
				break;
			case 'checkbox_group':
				echo '<fieldset><table rowspacing="0" cellspacing="0" border="0">';
				if ( ! empty( $field['options'] ) ) {
					$default = get_option( $field['id'], $field['default'] );
					
					if ( ! is_array( $default ) ) {
						$default = [ $default ];
					}
					
					$i = 0;
					foreach ($field['options'] as $key => $option) {
						if ( ++$i > 1 ) {
							//echo '<br />';
						}
						echo '<tr><td>' . $option['label'] . '</td><td><label for="' . esc_attr( $field['id'] ) . '-' . $i . '" class="check-switch">';
						echo '<input name="' . esc_attr( $field['id'] ) . '[' . $key . ']" id="' . esc_attr( $field['id'] ) . '-' . $i . '" type="checkbox" value="1"' . ( in_array( $key, $default ) || ! empty( $default[ $key ] ) ? ' checked' : '' ) . '><span class="slider"></span></label></td></tr>';
					}
				}
				if ( ! empty( $field['description'] ) ) {
					echo '<tr><td colspan="2"><p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p></td></tr>';
				}
				echo '</table></fieldset>';
				break;
			case 'image':
				$image_id = get_option($field['id']);
				$image_object = '';
				
				if ( ! empty( $image_id ) ) {
					$image_object = wp_get_attachment_image( $image_id, 'medium' );
				}
				
				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="hidden" value="' . esc_attr( $image_id ) . '">
				<style>#' . esc_attr( $field['id'] ) . '-preview .image img{margin-bottom: 10px;}</style>
				<div id="' . esc_attr( $field['id'] ) . '-preview"><div class="image">' . $image_object . '</div></div>
				<input class="upload-image button" type="button" data-id="' . esc_attr( $field['id'] ) . '" value="' . esc_attr__( 'Select Image', ZYNITH_SEO_TEXT_DOMAIN ) . '">
				<input class="remove-image button" type="button" data-id="' . esc_attr( $field['id'] ) . '" value="' . esc_attr__( 'Remove Image', ZYNITH_SEO_TEXT_DOMAIN ) . '"' . ( empty( $image_id ) ? ' style="display: none;"' : '' ) . '>';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'select':
				echo '<select name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" class="regular-text styled-select">';
				
				if ( ! empty( $field['options'] ) ) {
					$default = get_option( $field['id'], $field['default'] );
					
					foreach ( $field['options'] as $key => $value ) {
						echo '<option value="' . esc_attr( $key ) . '"' . ( (string) $key === (string) $default ? ' selected' : '' ) . '>' . $value . '</option>';
					}
				}
				
				echo '</select>';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'select_multi':
				echo '<select name="' . esc_attr( $field['id'] ) . '[]" id="' . esc_attr( $field['id'] ) . '" class="" multiple style="max-width: 470px; width: 100%;">';
				
				if (! empty( $field['options'] ) ) {
					$default = get_option( $field['id'], $field['default'] );
					
					if ( ! is_array( $default ) ) {
						$default = [ (string) $default ];
					}
					
					foreach ( $field['options'] as $key => $value ) {
						echo '<option value="' . esc_attr( $key ) . '"' . ( in_array( $key, $default ) || ! empty( $default[ $key ] ) ? ' selected' : '' ) . '>' . esc_html( $value ) . '</option>';
					}
				}
				
				echo '</select>';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'radio':
				echo '<fieldset>';
				
				if ( ! empty( $field['options'] ) ) {
					$default = get_option( $field['id'], $field['default'] );
					
					$i = 0;
					foreach ( $field['options'] as $key => $value ) {
						if ( ++$i > 1 ) {
							echo '<br>';
						}
						
						echo '<label for="' . esc_attr( $field['id'] ) . '-' . $i . '" class="check-switch"><input name="' . esc_attr( $field['id'] ) . '" id="' . $field['id'] . '-' . $i . '" type="' . esc_attr( $field['type'] ) . '" value="' . esc_attr( $key ) . '"' . ( $key === $default ? ' checked' : '' ) . '><span class="slider"></span></label>' . $value;
					}
				}
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				echo '</fieldset>';
				
				break;
			case 'color':
				$default = get_option( $field['id'], $field['default'] );
				
				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="text" value="' . esc_attr( $default ) . '" data-default-color="' . esc_attr( $default ) . '" class="regular-text color-picker">';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'textarea':
				$default = get_option( $field['id'], $field['default'] );
				
				echo '<textarea name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" rows="8" class="regular-text code" style="width: 100%">' . esc_html( $default ) . '</textarea>';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'number':
				$default = get_option( $field['id'], $field['default'] );
				
				$min = $field['min'] ?? '0';
				$max = $field['max'] ?? null;
				$step = $field['step'] ?? '1';
				$placeholder = $field['placeholder'] ?? '';
				
				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="' . esc_attr( $field['type'] ) . '" value="' . esc_attr( $default ) . '" placeholder="' . esc_attr( $placeholder ) . '" min="' . esc_attr( $min ) . '"' . ( ! empty( $max ) ? ' max="' . esc_attr( $max ) . '"' : '' ) . ' step="' . esc_attr( $step ) . '" class="regular-text">';
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'password':
				$default = esc_attr( get_option( $field['id'] ) );
				$placeholder = $field['placeholder'] ?? '';
				$purge_button = (bool) ( $field['purge_button'] ?? true );
				$purge_button_text = $field['purge_button_text'] ?? __( 'Purge', ZYNITH_SEO_TEXT_DOMAIN );
				$confirm_dialog = $field['confirm_dialog'] ?? __( 'Are you sure?', ZYNITH_SEO_TEXT_DOMAIN );
				
				$params = '';
				if ( ! empty( $default ) ) {
					$default = str_repeat( '*', mb_strlen( $default ) );
					$params = ' readonly="readonly" style="pointer-events: none"';
				}

				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="' . esc_attr( $field['type'] ) . '" value="' . $default . '" placeholder="' . esc_attr( $placeholder ) . '" class="regular-text" autocomplete="off"' . $params . '>';
				
				if ( ! empty( $default ) && $purge_button ) {
					echo '<a href="#" class="zynith-button purge" data-confirm="' . $confirm_dialog . '">' . $purge_button_text . '</a>';
				}

				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'editor':
				$default = get_option( $field['id'] );

				wp_editor( $default, esc_attr( $field['id'] ) );
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				break;
			case 'message':
				if ( ! empty( $field['description'] ) ) {
					echo '<div class="description" style="line-height: 20px;">' . $field['description'] . '</div>';
				}
				break;
			case 'text_group':
				echo '<fieldset>';
				
				if ( ! empty( $field['options'] ) ) {
					$saved = get_option( $field['id'], [] );
					
					$i = 0;
					foreach ( $field['options'] as $key => $value ) {
						$default = $saved[ $key ] ?? '';
						
						if ( ++$i > 1 ) {
							echo '<br>';
						}
						
						echo '<label for="' . esc_attr( $field['id'] ) . '-' . $i . '" class="text-group">' . $value . '</label>';
						echo '<input name="' . esc_attr( $field['id'] ) . '[' . $key . ']" id="' . esc_attr( $field['id'] ) . '-' . $i . '" type="text" value="' . esc_attr( $default ) . '" class="regular-text">';
					}
				}
				
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
				
				echo '</fieldset>';
				
				break;
			default:
				$default = get_option( $field['id'], $field['default'] );
				$placeholder = $field['placeholder'] ?? '';
				$required = ! empty( $field['required'] ) ? ' required' : '';

				echo '<input name="' . esc_attr( $field['id'] ) . '" id="' . esc_attr( $field['id'] ) . '" type="' . esc_attr( $field['type'] ) . '" value="' . esc_attr( $default ) . '" placeholder="' . esc_attr( $placeholder ) . '" class="regular-text" style="max-width: 470px; width: 100%;"' . $required . '>';
				if ( ! empty( $field['description'] ) ) {
					echo '<p class="description" id="' . esc_attr( $field['id'] ) . '-description">' . $field['description'] . '</p>';
				}
		}
	}

	public function enqueue_scripts() {
		$zynith_seo_assets = plugin_dir_url( ZYNITH_SEO_FILE ) . 'assets';
		$cdnjs_assets = 'https://cdnjs.cloudflare.com/ajax/libs';
		
        $selected_style = get_option( 'zynith_style_select', 'default' );
        $admin_stylesheet = 'admin';

        if ( $selected_style !== 'default' ) {
            $admin_stylesheet .= '-' . $selected_style;
        }

        $stylesheet_id = ZYNITH_SEO_TEXT_DOMAIN . '-' . $admin_stylesheet;
		
		wp_enqueue_media();
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style( 'zynith-style', $zynith_seo_assets . '/css/style.min.css', [], ZYNITH_SEO_VERSION );
		wp_enqueue_style( $stylesheet_id, $zynith_seo_assets . '/css/' . $admin_stylesheet . '.min.css', [], ZYNITH_SEO_VERSION );
		wp_enqueue_style( 'select2', $cdnjs_assets . '/select2/4.0.13/css/select2.min.css', false, ZYNITH_SEO_VERSION );
		wp_register_script( ZYNITH_SEO_TEXT_DOMAIN, $zynith_seo_assets . '/js/admin.min.js', [ 'jquery' ], ZYNITH_SEO_VERSION, true );
		wp_enqueue_script( 'select2', $cdnjs_assets . '/select2/4.0.13/js/select2.min.js', [ 'jquery' ], ZYNITH_SEO_VERSION, true );
		wp_enqueue_script( 'wp-color-picker' );
		wp_enqueue_script( ZYNITH_SEO_TEXT_DOMAIN );

		wp_localize_script( ZYNITH_SEO_TEXT_DOMAIN, 'custom_image', [
			'title' => __( 'Select Image', ZYNITH_SEO_TEXT_DOMAIN ),
			'button' => __( 'Select Image', ZYNITH_SEO_TEXT_DOMAIN ),
			'og_title' => __( 'Select Open Graph Image', ZYNITH_SEO_TEXT_DOMAIN ),
			'og_button' => __( 'Select Image', ZYNITH_SEO_TEXT_DOMAIN ),
			'logo_title' => __( 'Select Logo Image', ZYNITH_SEO_TEXT_DOMAIN ),
			'logo_button' => __( 'Select Image', ZYNITH_SEO_TEXT_DOMAIN ),
		] );

        wp_enqueue_script( ZYNITH_SEO_TEXT_DOMAIN );
        wp_localize_script( 'jquery', 'customAdminScript', array(
            'nonce' => wp_create_nonce( 'save_meta_data_nonce' ),
        ) );
	}

	public static function get_abspath() {
		$doc_root = filter_input( INPUT_SERVER, 'DOCUMENT_ROOT' );

		if ( empty( $doc_root ) ) {
			if ( wp_is_writable( ABSPATH ) ) {
				$doc_root = ABSPATH;
			} else {
				$doc_root = dirname( WP_CONTENT_DIR );
			}
		}

		return htmlspecialchars( rtrim( $doc_root, '/' ) ) . '/';
	}

	private function get_robots_txt_path() {
		return self::get_abspath() . 'robots.txt';
	}

	public static function get_sitemap_path() {
		return self::get_abspath() . self::get_sitemap_name();
	}

	private function update_robots_txt_file() {
		$use_dynamic_robots_txt = (bool) get_option( 'zynith_dynamic_robots_txt', false );
		$robots_txt_path = $this->get_robots_txt_path();

		@touch( $robots_txt_path );
		@chmod( $robots_txt_path, 0664 );

		if ( $use_dynamic_robots_txt ) {
			return unlink( $robots_txt_path );
		} else {
			$robots_txt_content = stripslashes_deep( get_option( 'zynith_robots_txt_content' ) );

			if ( wp_is_writable( $robots_txt_path ) ) {
				if ( @file_put_contents( $robots_txt_path, $robots_txt_content ) !== false ) {
					return true;
				}
			}
		}

		return false;
	}
	
	public function show_updated_notice() {
		if ( ! is_admin() ) {
			return;
		}
		
		$settings_pages = array_keys( $this -> settings );
		$page = filter_input( INPUT_GET, 'page' );
		if ( ! in_array( $page, $settings_pages ) ) {
			return;
		}

		$purged = filter_input( INPUT_GET, 'zynith-purged' );
        if ( ! empty( $purged ) ) {
            $class = 'notice notice-success is-dismissable';
            $message = __( 'Zynith SEO meta data purged', ZYNITH_SEO_TEXT_DOMAIN );
    
            printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
        }
				
		$updated = filter_input( INPUT_GET, 'settings-updated' );
		if ( empty( $updated ) ) {
			return;
		}

		$class = 'notice notice-success is-dismissable';
		
		if ( $page === 'zynith-seo-robots-txt' ) {
			if ( $this->update_robots_txt_file() ) {
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), __( 'Robots.txt file updated successfully.', ZYNITH_SEO_TEXT_DOMAIN ) );
			} else {
				$class = 'notice notice-error is-dismissable';
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), __( 'Robots.txt file could not be saved.', ZYNITH_SEO_TEXT_DOMAIN ) );
			}
		} else {
			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), __( 'Settings saved', ZYNITH_SEO_TEXT_DOMAIN ) );
		}

		flush_rewrite_rules();
	}

	public function show_sitemap_xml_warnings() {
		$sitemap_xml_error = get_option( 'sitemap_xml_error' );
		$sitemap_generated = filter_input( INPUT_GET, 'sitemap-generated' );

		if ( !empty( $sitemap_xml_error ) ) {
			$class = 'notice notice-warning';
			
			$dismiss_url = admin_url();
			$dismiss_url = add_query_arg( 'page', 'zynith-seo-settings', $dismiss_url );
			$dismiss_url = add_query_arg( 'action', 'dismiss-sitemap-xml', $dismiss_url );
			$dismiss_url = add_query_arg( '_wpnonce', wp_create_nonce( 'dismiss-sitemap-xml' ), $dismiss_url );
			
			$message = $sitemap_xml_error;
			$message .= '<div style="margin-top: 10px">';
			$message .= '<a href="' . $dismiss_url . '" class="zynith-button-small" style="margin-left: 8px;">' . __( "OK, I got it", ZYNITH_SEO_TEXT_DOMAIN ) . '</a>';
			$message .= '</div>';
            
			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $message );
		} else {
			if ( $sitemap_generated ) {
				$class = 'notice notice-success';
				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), __( "Sitemap was updated.", ZYNITH_SEO_TEXT_DOMAIN ) );
			}
		}
	}
	
}