<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class ZynithSEOBreadcrumbs {

    public $post_types;
    public $args;

    public function __construct() {
        add_action( 'template_redirect', [ $this, 'set_post_types' ], 1 );
		
		$this->args = apply_filters( 'zynith_breadcrumb_args', [
			'wrap_start' => '<div class="breadcrumbs">',
			'wrap_end' => '</div>',
			'separator' => ' <span class="sep">&raquo;</span> ',
		] );

        add_shortcode( 'zynith-breadcrumbs', [ $this, 'render_breadcrumbs' ] );
    }

    public static function init() {
        return new self();
    }

    public function set_post_types() {
        $this->post_types = array_keys( ZynithSEOPostTypes::get_post_types() );
    }

	public function render_breadcrumbs() {
		if ( is_admin() || is_front_page() ) {
			return;
		}

		$add_schema = (bool) get_option( 'zynith_add_breadcrumbs_schema', false );
		$schema_array = [ '@context' => 'https://schema.org', '@type' => 'BreadcrumbList', 'itemListElement' => [ [ '@type' => 'ListItem', 'position' => 1, 'item' => [ '@id' => home_url(), 'name' => _x( 'Home', 'Breadcrumbs Home', ZYNITH_SEO_TEXT_DOMAIN ), ], ] ], ];

		$custom_label = '';
		if (is_404()) {
			$custom_label = __( '404 Error - Page not found', ZYNITH_SEO_TEXT_DOMAIN );
		} else if (is_author()) {
			$custom_label = __( 'Author: ', ZYNITH_SEO_TEXT_DOMAIN ) . get_the_author();
		} else if (is_archive()) {
			$custom_label = __( 'Archive: ', ZYNITH_SEO_TEXT_DOMAIN ) . get_the_archive_title();
		}

		$breadcrumbs = $this->args['wrap_start'];
		$breadcrumbs .= '<a href="' . home_url() . '">' . _x( 'Home', 'Breadcrumbs Home', ZYNITH_SEO_TEXT_DOMAIN ) . '</a>' . $this->args['separator'];
		$uri = htmlspecialchars( trim( filter_input( INPUT_SERVER, 'REQUEST_URI' ), '/' ) );
		$url = ! empty( $uri ) ? explode( '/', $uri ) : [];

		if ( empty( array_filter( $url ) ) ) {
			return '';
		}
		
		$url = array_filter( $url );
		$breadcrumb_url = home_url( '/' );
		$item_position = 2;

		for ( $i = 0; $i < count( $url ); $i++ ) {
			$breadcrumb_url .= $url[ $i ] . '/';
			$label = mb_convert_case( trim( preg_replace( '%\s*[-_\s]+\s*%', ' ', $url[ $i ] ) ), MB_CASE_TITLE, "UTF-8" );
	
			if ( $i === count( $url ) ) {
				if ( ! empty( $custom_label ) ) {
					$label = $custom_label;
				}

				$breadcrumbs .= '<span class="current">' . $label . '</span>';
			} else {
				$breadcrumbs .= '<a href="' . esc_url( $breadcrumb_url ) . '">' . $label . '</a>' . $this->args['separator'];
			}

			$schema_array['itemListElement'][] = [ '@type' => 'ListItem', 'position' => $item_position, 'item' => [ '@id' => esc_url( $breadcrumb_url ), 'name' => $label, ], ];
			$item_position++;
		}

		$breadcrumbs .= $this->args['wrap_end'];

		if ( $add_schema ) {
			$breadcrumbs .= '<script type="application/ld+json">' . json_encode( $schema_array, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ) . '</script>';
		}

		return $breadcrumbs;
	}

}
