<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class ZynithQuickRyter {

    public function __construct() {
        $api_key = get_option( 'zynith_quick_ryter_api_key' );
        $show_metabox = (bool) get_option( 'zynith_show_quick_ryter', true );

        if ( empty( $api_key ) || ! $show_metabox ) {
            return;
        }
        
        add_action('add_meta_boxes', array($this, 'add_custom_meta_box'));
        add_action('save_post', array($this, 'save_custom_meta_box'), 10, 3);
        add_action('wp_ajax_my_ajax_handler', array($this, 'my_ajax_handler'));
        add_action('wp_ajax_nopriv_my_ajax_handler', array($this, 'my_ajax_handler'));
    }

        public static function init() {
        return new self();
    }

    public function generateText($prompt) {
        $url = 'https://api.openai.com/v1/engines/text-davinci-003/completions';
        $api_key = get_option('zynith_quick_ryter_api_key');

        $headers = array(
            'Content-Type: application/json',
            'Authorization: Bearer ' . $api_key
        );

        $fields = array(
            'prompt' => $prompt,
            'max_tokens' => 2000
        );

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fields));

        $response = curl_exec($ch);

        if (curl_errno($ch)) {
            echo 'Error: ' . curl_error($ch);
        } else {
            $response = json_decode($response, true);
            return $response['choices'][0]['text'];
        }

        curl_close($ch);
    }

    public function add_custom_meta_box() {
        add_meta_box('custom_meta_box', 'Quick Ryter Content Generator', [ $this, 'custom_meta_box_markup' ], [ 'post', 'page' ], 'normal', 'high', null);
    }

    public function custom_meta_box_markup($object) {
        wp_nonce_field(basename(__FILE__), "meta-box-nonce");

        $h1 = get_post_meta($object->ID, "meta-box-h1", true);
        $h2s = get_post_meta($object->ID, "meta-box-h2s", true);
        $brand_or_company_name = get_post_meta($object->ID, "meta-box-brand-name", true);

        echo '<div class="my-custom-class">';
        echo '<label for="meta-box-h1">' . __( 'Add your article title or H1:', ZYNITH_SEO_TEXT_DOMAIN ) . '</label><br>';
        echo '<input class="equal-width-input" id="meta-box-h1" name="meta-box-h1" type="text" value="' . $h1 . '">';
        echo '</div>';

        echo '<div class="my-custom-class">';
        echo '<label for="meta-box-h2s">' . __( 'Add your H2(s) as a comma-separated list:', ZYNITH_SEO_TEXT_DOMAIN ) . '</label><br>';
        echo '<input class="equal-width-input" id="meta-box-h2s" type="text" value="' . $h2s . '">';
        echo '</div>';

        echo '<div class="my-custom-class">';
        echo '<label for="meta-box-brand-name">' . __( 'Add your brand or business name:', ZYNITH_SEO_TEXT_DOMAIN ) . '</label><br>';
        echo '<input class="equal-width-input" id="meta-box-brand-name" name="meta-box-brand-name" type="text" value="' . $brand_or_company_name . '">';
        echo '</div>';

        ?>
        <div class="div_button_container"><button id="generate-button" class="button button-primary button-large"><?php _e( 'Generate Content', ZYNITH_SEO_TEXT_DOMAIN ) ?></button></div>
        <div id="loading-text" style="display: none;"><?php _e( 'Loading...', ZYNITH_SEO_TEXT_DOMAIN ) ?></div>
        <script>
            document.addEventListener('DOMContentLoaded', function () {
                var generateButton = document.getElementById('generate-button');
                var loadingText = document.getElementById('loading-text');

                generateButton.addEventListener('click', function (e) {
                    e.preventDefault();

                    var postID = document.getElementById('post_ID').value;
                    var h1 = document.getElementById('meta-box-h1').value;
                    var h2s = document.getElementById('meta-box-h2s').value;
                    var brandName = document.getElementById('meta-box-brand-name').value;
                    var nonce = '<?php echo wp_create_nonce('my_custom_nonce'); ?>';

                    generateButton.style.display = 'none';
                    loadingText.style.display = 'block';

                    var xhr = new XMLHttpRequest();
                    xhr.open('POST', '<?php echo admin_url('admin-ajax.php'); ?>', true);
                    xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                    xhr.onreadystatechange = function () {
                        if (xhr.readyState === 4 && xhr.status === 200) {
                            alert('<?php _e( 'Content generated successfully!', ZYNITH_SEO_TEXT_DOMAIN ) ?>');
                            location.reload();
                        }
                    };
                    xhr.send('action=my_ajax_handler&post_id=' + postID + '&h1=' + encodeURIComponent(h1) + '&h2s=' + encodeURIComponent(h2s) + '&brand_name=' + encodeURIComponent(brandName) + '&_ajax_nonce=' + encodeURIComponent(nonce));
                });
            });
        </script>
        <?php
    }

    public function save_custom_meta_box($post_id, $post, $update) {
        if (!isset($_POST["meta-box-nonce"]) || !wp_verify_nonce($_POST["meta-box-nonce"], basename(__FILE__)))
            return $post_id;

        if(!current_user_can("edit_post", $post_id))
            return $post_id;

        if(defined("DOING_AUTOSAVE") && DOING_AUTOSAVE)
            return $post_id;

        if(isset($_POST["meta-box-h1"])) {
            $h1 = sanitize_text_field($_POST["meta-box-h1"]);
            update_post_meta($post_id, "meta-box-h1", $h1);
        }

        if(isset($_POST["meta-box-h2s"])) {
            $h2s = sanitize_text_field($_POST["meta-box-h2s"]);
            update_post_meta($post_id, "meta-box-h2s", $h2s);
        }
        if(isset($_POST["meta-box-brand-name"])) {
            $brand_name = sanitize_text_field($_POST["meta-box-brand-name"]);
            update_post_meta($post_id, "meta-box-brand-name", $brand_name);
        }
    }

    public function my_ajax_handler() {
        check_ajax_referer('my_custom_nonce');

        $post_id = intval($_POST['post_id']);

        $h1 = sanitize_text_field($_POST['h1']);
        $h2s = array_map('sanitize_text_field', explode(',', $_POST['h2s']));
        $brand_name = sanitize_text_field($_POST['brand_name']);

        // Generate the introduction based on H2 topics
        $introPrompt = __( "Please write an introduction about the following topics ensuring to use compound sentence structure: ", ZYNITH_SEO_TEXT_DOMAIN );
        $introPrompt .= "'" . implode("', '", $h2s) . "'.";
        $introContent = $this->generateText($introPrompt);
        $content = "<h1>" . $h1 . "</h1>\n" . $introContent . "\n";

        foreach($h2s as $h2) {
            if (!empty($brand_name)) {
                $prompt = sprintf( __( "Please write an informative section about '%s' on the topic of '%s', ensuring to use longer compound sentences when possible. Make sure to use transitional phrases between paragraphs, choosing from 'However', 'Moreover', 'On the other hand', 'In addition', 'Therefore', 'Furthermore', 'For instance', 'In contrast', 'Similarly', 'As a result', and 'Consequently' while trying to use each transitional phrase only once. Please also do not use 'In conclusion' in text. ", ZYNITH_SEO_TEXT_DOMAIN ), $brand_name, $h2 );
            } else {
                $prompt = sprintf( __( "Please write an informative section on the topic of '%s', ensuring to use longer compound sentences when possible. Make sure to use transitional phrases between paragraphs, choosing from 'However', 'Moreover', 'On the other hand', 'In addition', 'Therefore', 'Furthermore', 'For instance', 'In contrast', 'Similarly', 'As a result', and 'Consequently' while trying to use each transitional phrase only once. Please also do not use 'In conclusion' in text.", ZYNITH_SEO_TEXT_DOMAIN ), $h2 );
            }
            $generated_content = $this->generateText($prompt);
            $content .= "<h2>" . $h2 . "</h2>\n" . $generated_content . "\n";
        }

        // Generate the conclusion based on H1 and H2 topics
        $conclusionPrompt = "Please write a comprehensive conclusion summarizing the detailed information discussed about ";
        $conclusionPrompt .= "'" . implode("' and '", $h2s) . "' topics, ensuring to use longer, compounded sentences.";
        $conclusionContent = $this->generateText($conclusionPrompt);
        $content .= "<h2>" . __( "Conclusion", ZYNITH_SEO_TEXT_DOMAIN ) . "</h2>\n" . $conclusionContent . "\n";

        remove_action('save_post', array($this, 'save_custom_meta_box'));

        wp_update_post([
            'ID' => $post_id,
            'post_content' => $content
        ]);

        add_action('save_post', array($this, 'save_custom_meta_box'), 10, 3);

        // Update the post's content
        wp_update_post([
            'ID' => $post_id,
            'post_content' => $content
        ]);

        // Remove the shortcode from the post content
        remove_filter('the_content', 'do_shortcode', 11);

        wp_send_json_success();
    }
}
