<?php

if (!defined('ABSPATH')) {
    exit;
}

class ZynithTOC {

    public $args;

    public function __construct() {
        $this->args = apply_filters('zynith_toc_args', [
            'wrap_start' => '<div class="zynith-toc">',
            'wrap_end' => '</div>',
        ]);

        // Add shortcode handler
        add_shortcode('zynith-toc', [$this, 'generate_toc']);

        // Add content filter only when the shortcode is present
        add_filter('the_content', [$this, 'maybe_add_ids_to_headings']);
    }

    public static function init() {
        return new self();
    }

    public function maybe_add_ids_to_headings($content) {
        
		// Check if the [zynith-toc] shortcode is present in the content
        if (has_shortcode($content, 'zynith-toc')) {
            $content = $this->add_ids_to_headings($content);
        }
        return $content;
    }

    public function generate_toc() {
        $post = get_post();
        if (empty($post)) {
            return;
        }

        $content = $this -> add_ids_to_headings($post->post_content);
        preg_match_all('/<h([1-6]).*?id="(.*?)".*?>(.*?)<\/h\1>/s', $content, $matches, PREG_SET_ORDER);

        if (empty($matches)) {
            return '';
        }
		
		$output = get_option('zynith_toc_css');

		if ($output === '') {
			$output = '<style>
.zynith-toc {
	border: 1px solid gray;
    padding: 20px;
}
.zynith-toc ul {
	margin: 5px 0;
	padding-left: 0;
	list-style-type: none;
}
.zynith-toc li {
	margin-bottom: 0.1em;
}
.zynith-toc .toc-h1 a {
	font-weight: bold;
    font-size: 1.6em;
}
.zynith-toc .toc-h2 { padding-left: 20px; }
.zynith-toc .toc-h3 { padding-left: 40px; }
.zynith-toc .toc-h4 { padding-left: 60px; }
.zynith-toc .toc-h5 { padding-left: 80px; }
.zynith-toc .toc-h6 { padding-left: 100px; }
</style>';
		}
		else {
			$output = '<style>' . get_option('zynith_toc_css') . '</style>';
		}
		
		$output .= $this->args['wrap_start'] . '<ul>';

		$current_level = 0;
        foreach ($matches as $match) {
            $level = intval($match[1]);
            $id = $match[2];
            $title = wp_strip_all_tags($match[3]);

            while ($current_level < $level) {
                $output .= '<ul>';
                $current_level++;
            }
            while ($current_level > $level) {
                $output .= '</ul>';
                $current_level--;
            }
			
			if (get_option('zynith_toc_heading_' . $level . '_enabled') == 1) {
				$output .= "<li class=\"toc-h{$level}\"><a href=\"#{$id}\">{$title}</a></li>";
			}
			
        }

        while ($current_level > 0) {
            $output .= '</ul>';
            $current_level--;
        }

        $output .= '</ul>' . $this->args['wrap_end'];

        return $output;
    }

    public function add_ids_to_headings($content) {
        $dom = new DOMDocument();
        @$dom->loadHTML(mb_convert_encoding($content, 'HTML-ENTITIES', 'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);

        $uniqueIds = [];
        foreach (['h1', 'h2', 'h3', 'h4', 'h5', 'h6'] as $tag) {
            foreach ($dom->getElementsByTagName($tag) as $heading) {
                $idBase = strtolower(trim(preg_replace('/[^a-zA-Z0-9]+/', '-', $heading -> textContent), '-'));
                $id = $idBase;
                $counter = 1;

                // Ensure the ID is unique
                while (in_array($id, $uniqueIds)) {
                    $id = $idBase . '-' . $counter;
                    $counter++;
                }

                $uniqueIds[] = $id;
                $heading->setAttribute('id', $id);
            }
        }

        return $dom->saveHTML();
    }
}

ZynithTOC::init();