<?php
if (!defined('ABSPATH')) { exit; }

class ZynithSEO404Monitor {

	public function __construct() {
        register_deactivation_hook(__FILE__, [$this, 'delete_404_log_table']);
        add_action('admin_menu', [$this, 'add_404_monitor_page']);
        add_action('template_redirect', [$this, 'log_404_errors']);
		add_action('admin_init', [$this, 'handle_delete_404_logs']);
    }
	
    public static function init() {
        return new self();
    }
	
    public function delete_404_log_table() {
        global $wpdb;

        $table_name = $wpdb->prefix . 'zynith_404_log';

        $wpdb->query("DROP TABLE IF EXISTS $table_name");
    }

    public function add_404_monitor_page() {
        add_submenu_page(
            'zynith-seo-settings',
            __('404 Monitor', ZYNITH_SEO_TEXT_DOMAIN),
            __('404 Monitor', ZYNITH_SEO_TEXT_DOMAIN),
            'manage_options',
            '404-monitor-settings',
            [$this, 'display_404_monitor_settings']
        );
    }

    public function display_404_monitor_settings() {
    	if (!current_user_can('manage_options')) {
            wp_die('You do not have permission to access this page.');
        }

        global $wpdb;
        $table_name = $wpdb->prefix . 'zynith_404_log';
        $per_page = 50;
        $current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;
        $offset = ($current_page - 1) * $per_page;
        $total_logs = $wpdb->get_var("SELECT COUNT(*) FROM $table_name");
        $total_pages = ceil($total_logs / $per_page);

        // Retrieve logs for the current page
        $logs = $wpdb->get_results("SELECT * FROM $table_name ORDER BY timestamp DESC LIMIT $per_page OFFSET $offset", ARRAY_A);

        echo '<div class="wrap">';
        echo '<h2>404 Monitor</h2>';

        if (empty($logs)) { echo '<p>No 404 errors have been logged yet.</p>'; }
		else {
			 echo '<form id="zynith-form-404" method="post" action="">';
			 echo '<table class="widefat" id="zynith-table-404">';
			 echo '<thead><tr><th><input type="checkbox" id="select-all" /> Select All</th><th class="sortable">Error URL</th><th class="sortable">User Agent</th><th class="sortable">Timestamp</th></tr></thead>';
			 echo '<tbody>';

			foreach ($logs as $log) {
				echo '<tr>';
				echo '<td><input type="checkbox" class="log-checkbox" name="logs[]" value="' . $log['id'] . '"></td>';
				echo '<td>' . esc_html($log['url']) . '</td>';
				echo '<td>' . esc_html($log['user_agent']) . '</td>';
				echo '<td>' . esc_html($log['timestamp']) . '</td>';
				echo '</tr>';
			}

			echo '</tbody>';
			echo '</table>';

            // Add pagination links
            $page_links = paginate_links(array(
                'base' => add_query_arg('paged', '%#%'),
                'format' => '',
                'prev_text' => '&laquo; Previous',
                'next_text' => 'Next &raquo;',
                'total' => $total_pages,
                'current' => $current_page,
            ));

            if ($page_links) {
                echo '<div class="tablenav">
                    <div class="tablenav-pages">' . $page_links . '</div>
                </div>';
            }

            echo '<input type="submit" class="button button-primary" name="delete_404_logs" value="Delete Selected 404(s)">';
            echo '</form>';
        }

        echo '</div>';
    }

    public function log_404_errors() {
        if (is_404()) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'zynith_404_log';
            $log = array(
                'url' => esc_url($_SERVER['REQUEST_URI']),
                'user_agent' => esc_html($_SERVER['HTTP_USER_AGENT'])
            );
            $wpdb->insert($table_name, $log);
        }
    }

    public function handle_delete_404_logs() {
        if (isset($_POST['delete_404_logs'])) {
            global $wpdb;
            $table_name = $wpdb->prefix . 'zynith_404_log';

            $logs_to_delete = isset($_POST['logs']) ? $_POST['logs'] : array();

            if (!empty($logs_to_delete)) {
                $ids = implode(',', array_map('intval', $logs_to_delete));
                $wpdb->query("DELETE FROM $table_name WHERE id IN ($ids)");
            }
        }
    }
	
}