<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class ZynithSEOContentUtility {

    public static function calculate_keyword_density( $content, $keyword ) {
        $keyword = strtolower( trim( $keyword ) );
        $content = strtolower( wp_strip_all_tags( $content ) );
        $word_count = str_word_count( $content );
        $keyword_count = substr_count( $content, $keyword );

        if ( $word_count < 1 ) {
            return 0;
        }

        return round( ( $keyword_count / $word_count ) * 100, 2 );
    }

    // TODO: Consider counting the post title as h1? Or switch to DomDocument instead
    public static function get_all_headings( $content ) {
        // Check for headings in the content
        preg_match_all( '/<h([1-6])[^>]*>(.*?)<\/h[1-6]>/ui', $content, $headings, PREG_SET_ORDER );
    
        // Array to store the grouped headings
        $grouped_headings = [];
        foreach ( $headings as $heading ) {
            $level = $heading[1];
            $text = $heading[2];
            $grouped_headings["h$level"][] = $text;
        }
    
        return $grouped_headings;
    }

    public static function check_missing_alt_tags( $content ) {
        preg_match_all( '/<img[^>]+>/i', $content, $img_tags );
        if ( empty( $img_tags[0] ) ) {
            return false;
        }
    
        foreach ( $img_tags[0] as $img_tag ) {
            preg_match('/alt="([^"]*)"/i', $img_tag, $alt_attr);
            if ( empty( $alt_attr ) || empty( $alt_attr[1] ) ) {
                return true;
            }
        }
    
        return false;
    }

    public static function is_content_ai_generated( $content ) {
    // Define regular expressions for various patterns
    $noun_regex = '/\b(?:[a-z]*ing|tion|ment|ness|ity|ty|sion|ence|ance|er|or)\b/i';
    $verb_regex = '/\b(?:[a-z]*ed|[a-z]*en|[a-z]*es)\b/i';
    $adjective_regex = '/\b(?:[a-z]*y|[a-z]*ous|[a-z]*ful|[a-z]*ish|[a-z]*ive|[a-z]*al|[a-z]*ic)\b/i';
    $repeat_phrase_regex = '/\b(\w+\s){2,}.*\1/i'; 
    $idioms = '/\b(as cold as ice|add insult to injury|at the drop of a hat|back to the drawing board|barking up the wrong tree|beat around the bush|best of both worlds|bite the bullet|break a leg|call it a day|cut corners|hit the nail on the head|kick the bucket|let the cat out of the bag|make a long story short|miss the boat|once in a blue moon|piece of cake|see eye to eye|sit on the fence|the ball is in your court|under the weather|hit the books|paint the town red|when pigs fly)\b/i';
    $years = '/\b((19|20)\d{2}|eighteen[ \-]?\d{2}s|nineteen[ \-]?\d{2}s|twenty[ \-]?first century|(eighteen|nineteen|twenty)[ \-]?((twenty|thirty|forty|fifty|sixty|seventy|eighty|ninety)[ \-]?(one|two|three|four|five|six|seven|eight|nine)?|ten|eleven|twelve|thirteen|fourteen|fifteen|sixteen|seventeen|eighteen|nineteen))\b/i';
    $contractions = "/\b(?:I'm|you're|he's|she's|it's|we're|they're|I've|you've|we've|they've|I'll|you'll|he'll|she'll|we'll|they'll|I'd|you'd|he'd|she'd|we'd|they'd|isn't|aren't|wasn't|weren't|haven't|hasn't|hadn't|won't|wouldn't|don't|doesn't|didn't|can't|couldn't|shouldn't|mightn't|mustn't)\b/i";
    $because_sentences = "/\b(because|however|therefore|furthermore|nevertheless|in addition|in fact|on the other hand) .+?[.?!]\b/i";
    $opinion_phrases = "/\b(in my opinion|in our opinion|i believe|i think|i feel|from my perspective|it seems to me|i would argue that|it is my understanding|i suppose|in my view|i suggest|i reckon|as far as i'm concerned|to my mind|as i see it|my belief is|i hold the view)\b/i";
    $special_chars = '/;|\\—/'; // Semi-colon or dash


   // Count the occurrences of each pattern in the content
    $nouns = preg_match_all( $noun_regex, $content );
    $adjectives = preg_match_all( $adjective_regex, $content );
    $repeated_phrases = preg_match_all( $repeat_phrase_regex, $content );
    $idioms_count = preg_match_all( $idioms, $content );
    $years_count = preg_match_all( $years, $content );
    $contractions_count = preg_match_all( $contractions, $content );
    $because_sentences_count = preg_match_all( $because_sentences, $content );
    $opinion_phrases_count = preg_match_all( $opinion_phrases, $content );
    $special_chars_count = preg_match_all( $special_chars, $content );

    // Calculate the ratio of adjectives to nouns in the content
    $ratio = ($nouns > 0) ? $adjectives / $nouns : 0;

    // Count total words
    $total_words = str_word_count( $content );

    // Calculate the percentage of contractions used
    $contractions_percentage = ($total_words > 0) ? ($contractions_count / $total_words) * 100 : 0;

    // Define thresholds
    $adjective_noun_ratio = 0.5;
    $repeat_phrase_threshold = 3;
    $idioms_threshold = 3;
    $years_threshold = 3;
    $contractions_percentage_threshold = 10; // Adjust as per your requirements

    // Check if content potentially has AI patterns
    $has_ai_patterns = (
        $ratio >= $adjective_noun_ratio ||
        $repeated_phrases >= $repeat_phrase_threshold ||
        $idioms_count >= $idioms_threshold ||
        $years_count >= $years_threshold ||
        $contractions_percentage < $contractions_percentage_threshold ||
        $special_chars_count > 0 // Add condition for special characters detection if desired
    );

    // Check if content has human patterns
    $has_human_patterns = (
        $because_sentences_count > 0 ||
        $opinion_phrases_count > 0
    );

    // If it has more AI patterns and fewer human patterns, we assume it's AI-generated
    return (bool) ($has_ai_patterns && !$has_human_patterns);
}

    public static function check_content_length( $content ) {
        $word_count = str_word_count( wp_strip_all_tags( $content ) );

        return (bool) ( $word_count >= 1400 );
    }

    public static function detect_broken_links( $content, $object ) {
        $key = get_class( $object );
        $broken_links = [];

        if ( $key === 'WP_Post' ) {
            $object_id = $object->ID;
            $option = 'broken_links_posts';
            $title = $object->post_title;
            $edit = admin_url( '/post.php?post=' . $object_id . '&action=edit' );
        } elseif ( $key === 'WP_Term' ) {
            $object_id = $object->term_id;
            $option = 'broken_links_terms';
            $title = $object->name;
            $edit = admin_url( '/term.php?taxonomy=' . $object->taxonomy . '&tag_ID=' . $object_id );
        } elseif ( $key === 'WP_User' ) {
            $object_id = $object->ID;
            $option = 'broken_links_authors';
            $title = $object->display_name;
            $edit = admin_url( '/user-edit.php?user_id=' . $object_id );
        } else {
            return $broken_links;
        }

        $broken_links_options = get_option( $option, [] );
        unset( $broken_links_options[ $object_id ] );

        preg_match_all( '/<a[^>]+href=["\'](.*?)["\']/', $content, $links );
        foreach ( $links[1] as $link ) {
			$response = wp_safe_remote_get( $link );
			if ( is_wp_error( $response ) || wp_remote_retrieve_response_code( $response ) === 404 ) {
            	$broken_links[] = $link;
        	}
			/*
            if ( wp_http_validate_url( $link ) !== false ) {
                if ( stripos( get_headers( $link )[0], '404 Not Found' ) !== false ) {
                    $broken_links[] = $link;
                }
            } else {
                $broken_links[] = $link;
            }
			*/
        }
		
        if ( ! empty( $broken_links ) ) {
            $broken_links_options[ $object_id ] = [ 'broken_links' => $broken_links, 'title' => $title, 'edit' => $edit, ];
        }

        update_option( $option, $broken_links_options );

        return $broken_links;
    }

    // Old version of link counter
    public static function check_link_ratios( $content ) {
        $internal_links_count = preg_match_all( '/<a[^>]*href=["\'](?:(?!mailto:|tel:|javascript:|#|.*\.(pdf|docx?|xlsx?|pptx?|zip|rar|jpg|jpeg|png|gif|bmp|mp3|mp4|wav|ogg|avi|mov|mkv))(?:https?:\/\/)?(?:www\.)?' . preg_quote( $_SERVER['HTTP_HOST'] ) . '[^"\']*)["\'][^>]*>/i', $content, $internal_matches );
        $external_links_count = preg_match_all( '/<a[^>]*href=["\']((?:(?!mailto:|tel:|javascript:|#|.*\.(pdf|docx?|xlsx?|pptx?|zip|rar|jpg|jpeg|png|gif|bmp|mp3|mp4|wav|ogg|avi|mov|mkv))(?:https?:\/\/)?(?:www\.)?)[^"\']+)["\'][^>]*>/i', $content, $external_matches );
        
        // Remove internal links from the external links array
        $external_links_count = $external_links_count - $internal_links_count;
        $word_count = str_word_count( wp_strip_all_tags( $content ) );
        $internal_link_ratio = $internal_links_count / ( $word_count / 1000 );
        $external_link_ratio = $external_links_count / ( $word_count / 1000 );
        $internal_green_light = $internal_links_count > 0 && $internal_links_count < 4 && $internal_link_ratio <= 3;
        $internal_red_light = $internal_links_count == 0 || ( $internal_links_count >= 4 && $internal_link_ratio > 3 );
        $external_green_light = $external_links_count > 0 && $external_links_count <= 3 && $external_link_ratio <= 3;
        $external_red_light = $external_links_count > 3 && $external_link_ratio > 3;
    
        return [
            'internal_green_light' => $internal_green_light,
            'internal_red_light' => $internal_red_light,
            'external_green_light' => $external_green_light,
            'external_red_light' => $external_red_light,
            'internal_links' => $internal_links_count,
            'external_links' => $external_links_count
        ];
    }

    // New version of link counter
    public static function check_link_counts( $content ) {
		
        if ( empty( $content ) || !class_exists('DOMDocument')) {
            return [
                'internal_red_light' => true,
                'external_red_light' => true,
                'internal_links' => 0,
                'external_links' => 0,
            ];
        }
        
        $home_url = home_url();
        $internal_links = 0;
        $external_links = 0;
		
        $doc = new DOMDocument();
		
		libxml_use_internal_errors( true );
        $doc->loadHTML( $content );
        libxml_clear_errors();
        $links = $doc->getElementsByTagName( 'a' );

        foreach ( $links as $link ) {
            $href = $link->getAttribute( 'href' );
            if (strpos($href, $home_url) === 0 || strpos($href, '/') === 0) { $internal_links++; }
            else { $external_links++; }
        }

        return [
            'internal_red_light' => (bool) ( $internal_links < 3 ),
            'external_red_light' => (bool) ( $external_links < 3 ),
            'internal_links' => $internal_links,
            'external_links' => $external_links,
        ];

    }

    public static function calculate_html_to_text_ratio( $content ) {
        if ( empty( $content ) ) {
            return 0;
        }
    
        // Calculate the ratio
        $html_length = strlen( $content );
        $text_length = strlen( strip_tags( $content ) );

        return round( ( $text_length / $html_length ) * 100, 2 );
    }

}