<?php
if (!defined('ABSPATH')) { exit; }

class ZynithSEOSearchAndReplace {
	
	public function __construct() {
        add_action('admin_menu', [$this, 'add_search_and_replace_pages' ], 99 );
    }
	
	public function add_search_and_replace_pages() {
        add_submenu_page( 'zynith-seo-settings', __( 'Search and Replace Content', ZYNITH_SEO_TEXT_DOMAIN ), __( 'Search & Replace', ZYNITH_SEO_TEXT_DOMAIN ), 'manage_options', 'search-and-replace', [ $this, 'search_and_replace_page' ] );
    }
	
	public static function init() {
        return new self();
    }
	
	public function search_and_replace_page() {
    	?>
		<div class="wrap zynith-wrap">
			<h1><?php _e( 'Search and Replace Content', ZYNITH_SEO_TEXT_DOMAIN ); ?></h1>
			<?php if (isset($_GET['content_updated'])) { ?>
			<div class="notice notice-success">
				<p><?php _e( "Your content has been updated. If you are using static caching, please refresh your website's cache to show the updated content.", ZYNITH_SEO_TEXT_DOMAIN ); ?></p>
			</div>
			<?php } ?>
			<div class="postbox zynith_seo">
				<div class="inside">
					<form method="post" action="<?php echo admin_url('admin-post.php'); ?>">
						<input type="hidden" name="action" value="process_search_and_replace_form">
						<?php wp_nonce_field('search_and_replace_nonce', 'search_and_replace_nonce_field'); ?>
						<table class="form-table">
							<tr>
								<th scope="row">
									<label for="old_text"><?php _e( 'Original text:', ZYNITH_SEO_TEXT_DOMAIN ); ?></label>
								</th>
								<td>
									<input name="old_text" type="text" id="old_text" class="regular-text" placeholder="Search" style="max-width: 470px; width: 100%;">
								</td>
							</tr>
							<tr>
								<th scope="row">
									<label for="new_text"><?php _e( 'Replacement text:', ZYNITH_SEO_TEXT_DOMAIN ); ?></label>
								</th>
								<td>
									<input name="new_text" type="text" id="new_text" class="regular-text" placeholder="Replace" style="max-width: 470px; width: 100%;">
								</td>
							</tr>
							<tr>
								<th scope="row">
									<label for="selected_pages" style="display: block;"><?php _e( 'Update pages/posts:', ZYNITH_SEO_TEXT_DOMAIN ); ?></label>
									<span style="display: block; margin: 4px 0; font-size: 12px; font-weight: 300; line-height: 16px;"><?php _e( "Select multiple posts or leave blank to update all pages and posts.", ZYNITH_SEO_TEXT_DOMAIN ); ?></span>
								</th>
								<td>
									<select name="selected_pages[]" id="selected_pages" multiple class="select2" style="max-width: 470px; width: 100%;">
										<?php
											$posts = $this->get_all_posts_by_type();
											foreach ($posts as $post_type => $settings) {
												echo '<optgroup label="' . $settings['label'] . '">';

												foreach ( $settings['posts'] as $post_id => $post_title ) {
													echo '<option value="' . $post_id . '">' . $post_title . '</option>';
												}

												echo '</optgroup>';
											}
										?>
									</select>
								</td>
							</tr>
						</table>
					<?php submit_button(__( 'Replace Content', ZYNITH_SEO_TEXT_DOMAIN )); ?>
					</form>
				</div>
			</div>
		</div>
    	<?php
	}
	
	public function search_and_replace_content($old_text, $new_text, $selected_pages) {
		global $wpdb;

		$posts_table = $wpdb->prefix . "posts";
		$selected_pages = array_map('intval', $selected_pages); // Sanitize page/post IDs

		if (!empty($selected_pages)) {
			$page_condition = " AND ID IN (" . implode(',', $selected_pages) . ")";
		} else {
			$page_condition = '';
		}

		$wpdb->query( $wpdb->prepare( "UPDATE $posts_table SET post_content = REPLACE(post_content, %s, %s) WHERE (post_type = 'post' OR post_type = 'page')$page_condition", $old_text, $new_text ) );
	}

	// Process the form and perform the search and replace when the form is submitted
	public function process_search_and_replace_form() {
		if (isset($_POST['old_text']) && isset($_POST['new_text'])) {
			$old_text = sanitize_text_field($_POST['old_text']);
			$new_text = sanitize_text_field($_POST['new_text']);
			$selected_pages = isset($_POST['selected_pages']) ? $_POST['selected_pages'] : [];
			$this->search_and_replace_content( $old_text, $new_text, $selected_pages );

			// After the replacement, redirect back to the form page with a success message
			wp_redirect(add_query_arg(['page' => 'search-and-replace', 'content_updated' => 'true'], admin_url('admin.php')));
			exit;
		}
	}

	public function get_all_posts_by_type() {
		global $wpdb;

		$post_types = ZynithSEOPostTypes::get_post_types();
		$posts_by_type = [];

		foreach ($post_types as $post_type => $settings) {
			$results = $wpdb->get_results( $wpdb->prepare( "SELECT `ID`, `post_title` FROM `{$wpdb->base_prefix}posts` WHERE `post_type` = %s AND NOT `post_status` = 'auto-draft'", $post_type ) );

			if ( empty( $results ) ) {
				continue;
			}
			
			$posts = [];
			foreach ($results as $result) {
            	$posts[$result->ID] = $result->post_title;
        	}

			$posts_by_type[$post_type] = [
            	'label' => $settings['label'],
            	'posts' => $posts,
        	];
		}

		return $posts_by_type;
	}
}
