<?php
/**
 * Plugin Name: Z Y N I T H
 * Plugin URI: https://zynith.app/pricing/
 * Description: Your one-stop plugin for elevating search engine rankings, driving traffic, and boosting conversions with streamlined on-page optimization tools and automatic sitemap updates!
 * Version: 6.2.8
 * Author: 20script
 * Author URI: https://www.20script.ir
 * License: GPL-3.0
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 * Tested up to: 6.4.1
 * Donate link: https://www.paypal.com/donate/?hosted_button_id=XVXQ3RX7N4SQN
 * Contributors: Schieler Mew, Kellie Watson, Daniel Nielsen
 * Tags: plugin, WordPress, SEO, search engine optimization, meta tags, XML sitemap, breadcrumbs, schema markup
 * Support: https://www.facebook.com/groups/761871078859984
 * Warning: Touching the plugin source code could lead to catastrophic failure!
 */

if (!defined('ABSPATH')) { exit; }

// Define constants and update version number
define('ZYNITH_SEO_VERSION', '6.2.8' );
define('ZYNITH_SEO_FILE', __FILE__);
define('ZYNITH_SEO_SLUG', plugin_basename(__DIR__));
define('ZYNITH_SEO_NAME', 'Z Y N I T H');
define('ZYNITH_SEO_UPDATE_URL', 'https://zynith.app/wp-json/zynith/v1/update-info/');
define('ZYNITH_SEO_TEXT_DOMAIN', 'zynith-seo');
define('ZYNITH_SEO_PLUGIN_URI', 'https://zynith.app/');
define('ZYNITH_SEO_IS_WOOCOMMERCE', (bool)class_exists('WooCommerce'));

load_plugin_textdomain(ZYNITH_SEO_TEXT_DOMAIN, false, dirname(plugin_basename(__FILE__)) . '/lang/');

require_once(__DIR__ . '/includes/init.php');

// Add action links to the plugin page
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'zynith_seo_action_links');
function zynith_seo_action_links($links) {
    $links[] = '<a href="' . esc_url(admin_url('admin.php?page=zynith-seo-settings')) . '">' . __('Settings', ZYNITH_SEO_TEXT_DOMAIN) . '</a>';
    return $links;
}

// Add meta links to the plugin
add_filter('plugin_row_meta', 'zynith_seo_meta_links', 10, 2);
function zynith_seo_meta_links($plugin_meta, $plugin_file) {
    if ($plugin_file === plugin_basename(ZYNITH_SEO_FILE)) {
        $plugin_meta = [
            '<a href="' . ZYNITH_SEO_PLUGIN_URI . '" target="_blank">' . __('Version', ZYNITH_SEO_TEXT_DOMAIN) . ' ' . ZYNITH_SEO_VERSION . '</a>',
            __('By', ZYNITH_SEO_TEXT_DOMAIN) . ' <a href="' . esc_url(ZYNITH_SEO_PLUGIN_URI) . '" target="_blank">Z Y N I T H</a>',
            '<a href="#" class="zynith-seo-view-details">' . __('View Details', ZYNITH_SEO_TEXT_DOMAIN) . '</a>',
        ];
    }

    return $plugin_meta;
}

// JavaScript to open the readme.txt file in a modal popup
function zynith_seo_modal_popup_script() {
    ?>
    <script>
        jQuery(document).ready(function($) {
            $('.zynith-seo-view-details').on('click', function(e) {
                e.preventDefault();
                var url = '<?php echo esc_url(plugin_dir_url(__FILE__) . 'readme.txt'); ?>';
                tb_show('Plugin Details', url);
				
				var tb = document.getElementById('TB_ajaxContent');
                tb.setAttribute( 'style', 'width: calc(100% - 30px); height: calc(100% - 50px);' );
            });
        });
    </script>
    <?php
}
add_action('admin_footer', 'zynith_seo_modal_popup_script');

// Initialize classes
ZynithSEOSettings::init();
ZynithSEOUpdater::init();
ZynithSEOCategoryBase::init();
ZynithSEOSchema::init();
ZynithSEORemoveGenesisSEO::init();
ZynithSEOSearchAndReplace::init();
ZynithSEO404Monitor::init();
ZynithSEOWPMods::init();
ZynithSEOPostTypes::init();
ZynithSEOMetaBoxes::init();
ZynithSEOTaxonomies::init();
ZynithSEOAuthors::init();
ZynithSEOImages::init();
ZynithSEOImport::init();
ZynithSEOSitemap::init();
ZynithSEOContent::init();
ZynithSEOBreadcrumbs::init();
ZynithTOC::init();
ZynithQuickRyter::init();